namespace EsbDecimals
{

#if (DEBUG)

	public class CompareOps
#else
	/// <summary>
	/// <para>Math Comparison Routines for <see cref="EsbDecimals"/> v2.1.0 - 10 June 2011.</para>
	/// <para>Equality, inequality, greater than and less than comparisons should
	/// not be done with the operators, rather use these methods where the 
	/// Precision of the comparison can be controlled.</para>
	/// <para>Methods for double, float and decimal included.</para>
	/// </summary>
	public static class CompareOps
#endif
	{
		#region Comparison Constants

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Float (1.0e-5f).
		/// </summary>
		public const float FloatPrecisionDef = 1.0e-5f;

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Double (1.0e-12).
		/// </summary>
		public const double DoublePrecisionDef = 1.0e-12;

		/// <summary>
		/// Default Precision when doing Floating Point Comparisons of type Decimal (1.0e-23m).
		/// </summary>
		public const decimal DecimalPrecisionDef = 1.0e-23m;

		#endregion //Comparison Constants

		#region GetPrecision Methods

		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="DoublePrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		public static double GetPrecision (double X, double Y)
		{
			if ((X == 0.0) || (Y == 0.0))
			{
				return DoublePrecisionDef;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * DoublePrecisionDef);
		}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision Value to use - should be small and positive like <see cref="DoublePrecisionDef"/>.</param>
		public static double GetPrecision (double X, double Y, double Precision)
		{
			if ((X == 0.0) || (Y == 0.0))
			{
				return Precision;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * Precision);
		}

		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="FloatPrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		public static float GetPrecision (float X, float Y)
		{
			if ((X == 0.0f) || (Y == 0.0f))
			{
				return FloatPrecisionDef;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * FloatPrecisionDef);
		}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision Value to use - should be small and positive like <see cref="FloatPrecisionDef"/>.</param>
		public static float GetPrecision (float X, float Y, float Precision)
		{
			if ((X == 0.0f) || (Y == 0.0f))
			{
				return Precision;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * Precision);
		}

		/// <summary>
		/// Returns the Precision Value to use using the Default Precision.
		/// See <see cref="DecimalPrecisionDef"/>.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		public static decimal GetPrecision (decimal X, decimal Y)
		{
			if ((X == 0.0m) || (Y == 0.0m))
			{
				return DecimalPrecisionDef;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * DecimalPrecisionDef);
		}

		/// <summary>
		/// Returns the Precision Value to use using the specified Precision
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision Value to use - should be small and positive like <see cref="DecimalPrecisionDef"/>.</param>
		public static decimal GetPrecision (decimal X, decimal Y, decimal Precision)
		{
			if ((X == 0.0m) || (Y == 0.0m))
			{
				return Precision;
			}
			return (System.Math.Min (System.Math.Abs (X), System.Math.Abs (Y)) * Precision);
		}

		#endregion //GetPrecision Methods

		#region SameValue Methods

		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(double,double)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to compare.</param>
		/// <param name="Y">Second Value to compare.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (double X, double Y)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y));
		}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(double,double,double)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (double X, double Y, double Precision)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y, Precision));
		}

		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(float,float)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to compare.</param>
		/// <param name="Y">Second Value to compare.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (float X, float Y)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y));
		}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(float,float,float)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (float X, float Y, float Precision)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y, Precision));
		}

		/// <summary>
		/// Returns True if the Values are within a "small" value of each other.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(decimal,decimal)"/></para>
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to compare.</param>
		/// <param name="Y">Second Value to compare.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (decimal X, decimal Y)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y));
		}

		/// <summary>
		/// Returns True if the Values are within a "small" value computed from the supplied Precision.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you want more control over the comparison then use <see cref="CompareValue(decimal,decimal,decimal)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if values are within Precision of each other.</returns>
		public static bool SameValue (decimal X, decimal Y, decimal Precision)
		{
			return (System.Math.Abs (X - Y) <= GetPrecision (X, Y, Precision));
		}

		#endregion //SameValue Methods

		#region CompareValue Methods

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(double,double)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (double X, double Y)
		{
			if (SameValue (X, Y))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(double,double,double)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (double X, double Y, double Precision)
		{
			if (SameValue (X, Y, Precision))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(float,float)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (float X, float Y)
		{
			if (SameValue (X, Y))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(float,float,float)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (float X, float Y, float Precision)
		{
			if (SameValue (X, Y, Precision))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(decimal,decimal)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (decimal X, decimal Y)
		{
			if (SameValue (X, Y))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		/// <summary>
		/// Returns 0 if Values are within a "small" value computed from the supplied Precision, 
		/// 1 if the first value is greater than Y and -1 if the first value is lesser.
		/// For Values with lots of significant figures, then Precision should be made smaller.
		/// <para>If you just want to know whether two values are the same then use <see cref="SameValue(decimal,decimal,decimal)"/></para>
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>0 (Same), 1 (First Value Greater) or -1 (Second Value Greater).</returns>
		public static int CompareValue (decimal X, decimal Y, decimal Precision)
		{
			if (SameValue (X, Y, Precision))
			{
				return 0;
			}
			if (X > Y)
			{
				return 1;
			}
			return -1;
		}

		#endregion //CompareValue Methods

		#region GreaterValue Methods

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (double X, double Y)
		{
			return (X - Y > GetPrecision (X, Y));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (double X, double Y, double Precision)
		{
			return (X - Y > GetPrecision (X, Y, Precision));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (float X, float Y)
		{
			return (X - Y > GetPrecision (X, Y));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (float X, float Y, float Precision)
		{
			return (X - Y > GetPrecision (X, Y, Precision));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (decimal X, decimal Y)
		{
			return (X - Y > GetPrecision (X, Y));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is greater.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if First Value is Greater.</returns>
		public static bool GreaterValue (decimal X, decimal Y, decimal Precision)
		{
			return (X - Y > GetPrecision (X, Y, Precision));
		} 

		#endregion //GreaterValue Methods

		#region LesserValue Methods

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (double X, double Y)
		{
			return (X - Y < -GetPrecision (X, Y));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (double X, double Y, double Precision)
		{
			return (X - Y < -GetPrecision (X, Y, Precision));
		}

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (float X, float Y)
		{
			return (X - Y < -GetPrecision (X, Y));
		} 

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (float X, float Y, float Precision)
		{
			return (X - Y < -GetPrecision (X, Y, Precision));
		}

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (decimal X, decimal Y)
		{
			return (X - Y < -GetPrecision (X, Y));
		}

		/// <summary>
		/// Returns True if Values are not within a "small" value of each other, and the First Value is lesser.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">First Value to Compare.</param>
		/// <param name="Y">Second Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if First Value is Lesser.</returns>
		public static bool LesserValue (decimal X, decimal Y, decimal Precision)
		{
			return (X - Y < -GetPrecision (X, Y, Precision));
		}

		#endregion //LesserValue Methods

		#region ValueIsZero Methods

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (double X)
		{
			return (System.Math.Abs (X) <= DoublePrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (double X, double Precision)
		{
			return (System.Math.Abs (X) <= Precision);
		}

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (float X)
		{
			return (System.Math.Abs (X) <= FloatPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (float X, float Precision)
		{
			return (System.Math.Abs (X) <= Precision);
		}

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (decimal X)
		{
			return (System.Math.Abs (X) <= DecimalPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is within a "small" value of 0.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if Value is very close to Zero.</returns>
		public static bool ValueIsZero (decimal X, decimal Precision)
		{
			return (System.Math.Abs (X) <= Precision);
		}

		#endregion //ValueIsZero Methods

		#region ValueIsPositive

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (double X)
		{
			return (X > DoublePrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if X is strictly Positive.</returns>
		public static bool ValueIsPositive (double X, double Precision)
		{
			return (X > Precision);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="FloatPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (float X)
		{
			return (X > FloatPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="FloatPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="FloatPrecisionDef"/>.</param>
		/// <returns>True if X is strictly Positive.</returns>
		public static bool ValueIsPositive (float X, float Precision)
		{
			return (X > Precision);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// <see cref="DecimalPrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Positive.</returns>
		public static bool ValueIsPositive (decimal X)
		{
			return (X > DecimalPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Positive.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DecimalPrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DecimalPrecisionDef"/>.</param>
		/// <returns>True if X is strictly Positive.</returns>
		public static bool ValueIsPositive (decimal X, decimal Precision)
		{
			return (X > Precision);
		}

		#endregion //ValueIsPositive

		#region ValueIsNegative Methods

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (double X)
		{
			return (X < -DoublePrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if X is strictly Negative.</returns>
		public static bool ValueIsNegative (double X, double Precision)
		{
			return (X < -Precision);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (float X)
		{
			return (X < -FloatPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if X is strictly Negative.</returns>
		public static bool ValueIsNegative (float X, float Precision)
		{
			return (X < -Precision);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// <see cref="DoublePrecisionDef"/> defines the Precision.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version that allows you to specify the Precision of the comparison.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <returns>True if Value is Negative.</returns>
		public static bool ValueIsNegative (decimal X)
		{
			return (X < -DecimalPrecisionDef);
		}

		/// <summary>
		/// Returns True if Value is not within a "small" value of 0 and is Negative.
		/// </summary>
		/// <remarks>
		/// There is an overloaded version without the Precision that will use
		/// <see cref="DoublePrecisionDef"/> for the Precision value.
		/// </remarks>
		/// <param name="X">Value to Compare.</param>
		/// <param name="Precision">Precision to used for comparison, should be positive and small,
		/// see <see cref="DoublePrecisionDef"/>.</param>
		/// <returns>True if X is strictly Negative.</returns>
		public static bool ValueIsNegative (decimal X, decimal Precision)
		{
			return (X < -Precision);
		}

		#endregion //ValueIsNegative Methods

	}
}



